//
//  FromAdderssCell.swift
//  SwiftFrom
//
//  Created by 刘春雷 on 2019/9/9.
//  Copyright © 2019 刘春雷. All rights reserved.
//

import UIKit

open class FromAdderssCell: FormValueCell, UIPickerViewDelegate, UIPickerViewDataSource {
    
    // MARK: Properties
    
    private let picker = KLCityPickerView()
    private let hiddenTextField = UITextField(frame: CGRect.zero)
    
    var myProvince : String?
    fileprivate var myCity : String?
    fileprivate var myArea : String?
    
    // MARK: FormBaseCell
    
    open override func configure() {
        super.configure()
        accessoryType = .none
        
        
//        picker.delegate = self
//        picker.dataSource = self
        hiddenTextField.inputView = picker
        picker.backgroundColor = UIColor.red
        hiddenTextField.isAccessibilityElement = false

        contentView.addSubview(hiddenTextField)
    }
    
    open override func update() {
        super.update()
        
        if let showsInputToolbar = rowDescriptor?.configuration.cell.showsInputToolbar , showsInputToolbar && hiddenTextField.inputAccessoryView == nil {
            hiddenTextField.inputAccessoryView = inputAccesoryView()
        }
        
        titleLabel.text = rowDescriptor?.title
        
        
        if let date = rowDescriptor?.value as? String {
            
            valueLabel.text = date
        }
        
    }
    
    open override func firstResponderElement() -> UIResponder? {
        return hiddenTextField
    }
    
    open override class func formViewController(_ formViewController: FormViewController, didSelectRow selectedRow: FormBaseCell) {
        guard let row = selectedRow as? FromAdderssCell else { return }
        row.picker.areaPickerViewWithProvince(province:row.myProvince , city: row.myCity, area: row.myArea) { (province, city, area) in
            if row.rowDescriptor?.value == nil {
                if let rowType = row.rowDescriptor?.type {
                    switch rowType {
                    case .address:
                        row.valueLabel.text = province+city+area
                        row.rowDescriptor?.value = province+city+area as AnyObject
                        break
                        
                    case .myProvince:
                        row.valueLabel.text = province
                        row.rowDescriptor?.value = province as AnyObject
                        break
                    case .ProvinceAndCity:
                        row.valueLabel.text = province+city
                        row.rowDescriptor?.value = province+city as AnyObject
                        break
                    default:
                        row.valueLabel.text = city
                        row.rowDescriptor?.value = city as AnyObject
                        break
                        
                    }
                }
                
                row.update()
            }

            print("\(province)\(city)\(area)")
        }
    }
    
    // MARK: UIPickerViewDelegate
    
    open func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        guard let options = rowDescriptor?.configuration.selection.options , !options.isEmpty else { return nil }
        guard row < options.count else { return nil }
        return rowDescriptor?.configuration.selection.optionTitleClosure?(options[row])
    }
    
    open func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int) {
        guard let options = rowDescriptor?.configuration.selection.options , !options.isEmpty else { return }
        guard row < options.count else { return }
        let newValue = options[row]
        rowDescriptor?.value = newValue
        valueLabel.text = rowDescriptor?.configuration.selection.optionTitleClosure?(newValue)
    }
    
    // MARK: UIPickerViewDataSource
    
    open func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    open func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        guard let options = rowDescriptor?.configuration.selection.options else { return 0 }
        return options.count
    }
}


